/*----------------------------------------------------------------------------*
*
*  mat4mul.c  -  4 x 4 matrix operations
*
*-----------------------------------------------------------------------------*
*
*  Copyright  2012 Hanspeter Winkler
*
*  This software is distributed under the terms of the GNU General Public
*  License version 3 as published by the Free Software Foundation.
*
*----------------------------------------------------------------------------*/

#include "mat4.h"


/* functions */

extern Status Mat4Mul
              (Coord A[4][4],
               Coord B[4][4],
               Coord C[4][4])
{
  Coord c00  =  A[0][0] * B[0][0]  +  A[0][1] * B[1][0]  +  A[0][2] * B[2][0]  +  A[0][3] * B[3][0];
  Coord c01  =  A[0][0] * B[0][1]  +  A[0][1] * B[1][1]  +  A[0][2] * B[2][1]  +  A[0][3] * B[3][1];
  Coord c02  =  A[0][0] * B[0][2]  +  A[0][1] * B[1][2]  +  A[0][2] * B[2][2]  +  A[0][3] * B[3][2];
  Coord c03  =  A[0][0] * B[0][3]  +  A[0][1] * B[1][3]  +  A[0][2] * B[2][3]  +  A[0][3] * B[3][3];

  Coord c10  =  A[1][0] * B[0][0]  +  A[1][1] * B[1][0]  +  A[1][2] * B[2][0]  +  A[1][3] * B[3][0];
  Coord c11  =  A[1][0] * B[0][1]  +  A[1][1] * B[1][1]  +  A[1][2] * B[2][1]  +  A[1][3] * B[3][1];
  Coord c12  =  A[1][0] * B[0][2]  +  A[1][1] * B[1][2]  +  A[1][2] * B[2][2]  +  A[1][3] * B[3][2];
  Coord c13  =  A[1][0] * B[0][3]  +  A[1][1] * B[1][3]  +  A[1][2] * B[2][3]  +  A[1][3] * B[3][3];

  Coord c20  =  A[2][0] * B[0][0]  +  A[2][1] * B[1][0]  +  A[2][2] * B[2][0]  +  A[2][3] * B[3][0];
  Coord c21  =  A[2][0] * B[0][1]  +  A[2][1] * B[1][1]  +  A[2][2] * B[2][1]  +  A[2][3] * B[3][1];
  Coord c22  =  A[2][0] * B[0][2]  +  A[2][1] * B[1][2]  +  A[2][2] * B[2][2]  +  A[2][3] * B[3][2];
  Coord c23  =  A[2][0] * B[0][3]  +  A[2][1] * B[1][3]  +  A[2][2] * B[2][3]  +  A[2][3] * B[3][3];

  Coord c30  =  A[3][0] * B[0][0]  +  A[3][1] * B[1][0]  +  A[3][2] * B[2][0]  +  A[3][3] * B[3][0];
  Coord c31  =  A[3][0] * B[0][1]  +  A[3][1] * B[1][1]  +  A[3][2] * B[2][1]  +  A[3][3] * B[3][1];
  Coord c32  =  A[3][0] * B[0][2]  +  A[3][1] * B[1][2]  +  A[3][2] * B[2][2]  +  A[3][3] * B[3][2];
  Coord c33  =  A[3][0] * B[0][3]  +  A[3][1] * B[1][3]  +  A[3][2] * B[2][3]  +  A[3][3] * B[3][3];

  C[0][0] = c00;  C[0][1] = c01;  C[0][2] = c02;  C[0][3] = c03;
  C[1][0] = c10;  C[1][1] = c11;  C[1][2] = c12;  C[1][3] = c13;
  C[2][0] = c20;  C[2][1] = c21;  C[2][2] = c22;  C[2][3] = c23;
  C[3][0] = c30;  C[3][1] = c31;  C[3][2] = c32;  C[3][3] = c33;

  return E_NONE;

}
